@@ -1,12 +1,12 @@
 
 /*
  *  defDirect.c
- *  $Id: defDirect.c,v 1.18 2001/09/29 20:40:59 bkorb Exp $
+ *  $Id: defDirect.c,v 1.19 2002/01/31 04:07:05 bkorb Exp $
  *  This module processes definition file directives.
  */
 
 /*
- *  AutoGen copyright 1992-1999 Bruce Korb
+ *  AutoGen copyright 1992-2002 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -28,8 +28,9 @@
 #include "directive.h"
 
 
-tSCC zNoEndif[]   = "ERROR:  in %s, #endif not found\n";
-tSCC zNoMatch[]   = "ERROR:  in %s, #%s found without an #ifdef/#ifndef\n";
+tSCC zNoEndif[]   = "Definition error:  in %s line %d, #endif not found\n";
+tSCC zNoMatch[]   = "Definition error:  in %s line %d, "
+                    "#%s no matching start/if directive\n";
 tSCC zCheckList[] = "\n#";
 
 STATIC int  ifdefLevel = 0;
@@ -48,7 +49,7 @@ STATIC char* skipToElseEnd( char* pzScan );
  *  Decide what to do and return a pointer to the character
  *  where scanning is to resume.
  */
-    EXPORT char*
+EXPORT char*
 processDirective( char* pzScan )
 {
     const tDirTable* pTbl  = dirTable;
@@ -62,7 +63,7 @@ processDirective( char* pzScan )
     for (;;) {
         pzEnd = strchr( pzScan, '\n' );
 
-        if (pzEnd == (char*)NULL) {
+        if (pzEnd == NULL) {
             /*
              *  The end of the directive is the end of the string
              */
@@ -87,6 +88,13 @@ processDirective( char* pzScan )
         pzEnd[-1] = pzEnd[0] = ' ';
     }
 
+    /*
+     *  Ignore ``#!'' as a comment, enabling a definition file to behave
+     *  as a script that gets interpreted by autogen.  :-)
+     */
+    if (*pzScan == '!')
+        return pzEnd;
+
     /*
      *  Find the start of the directive name
      */
@@ -136,7 +144,7 @@ processDirective( char* pzScan )
  *  count if it is a bogus name.
  */
 
-    STATIC teDirectives
+STATIC teDirectives
 findDirective( char* pzDirName )
 {
     teDirectives res = (teDirectives)0;
@@ -160,7 +168,7 @@ findDirective( char* pzDirName )
             ch = NUL;
         }
 
-        fprintf( stderr, "WARNING:  in %s on line %d unknown directive:\n"
+        fprintf( pfTrace, "WARNING:  in %s on line %d unknown directive:\n"
                  "\t#%s\n", pCurCtx->pzFileName, pCurCtx->lineNo, pzDirName );
 
         if (ch != NUL)
@@ -180,7 +188,7 @@ findDirective( char* pzDirName )
  *  accepting the preceeding text) or when encountering a "#if*def"
  *  while skipping a block of text due to a failed test.
  */
-    STATIC char*
+STATIC char*
 skipToEndif( char* pzScan )
 {
     for (;;) {
@@ -194,10 +202,9 @@ skipToEndif( char* pzScan )
             pz = ++pzScan;
         else {
             pz = strstr( pzScan, zCheckList );
-            if (pz == (char*)NULL) {
-                fprintf( stderr, zNoEndif, pCurCtx->pzFileName );
-                AG_ABEND;
-            }
+            if (pz == NULL)
+                AG_ABEND( asprintf( zNoEndif, pCurCtx->pzFileName,
+                                    pCurCtx->lineNo ));
 
             pzScan = pz + STRSIZE( zCheckList );
         }
@@ -211,7 +218,7 @@ skipToEndif( char* pzScan )
              *  We found the endif we are interested in
              */
             char* pz = strchr( pzScan, '\n' );
-            if (pz != (char*)NULL)
+            if (pz != NULL)
                 return pz+1;
             return pzScan + strlen( pzScan );
         }
@@ -241,7 +248,7 @@ skipToEndif( char* pzScan )
  *  "#elif*def".  We do this when we are skipping code due to a failed
  *  "#if*def" test.
  */
-    STATIC char*
+STATIC char*
 skipToElseEnd( char* pzScan )
 {
     for (;;) {
@@ -255,10 +262,9 @@ skipToElseEnd( char* pzScan )
             pz = ++pzScan;
         else {
             pz = strstr( pzScan, zCheckList );
-            if (pz == (char*)NULL) {
-                fprintf( stderr, zNoEndif, pCurCtx->pzFileName );
-                AG_ABEND;
-            }
+            if (pz == NULL)
+                AG_ABEND( asprintf( zNoEndif, pCurCtx->pzFileName,
+                                    pCurCtx->lineNo ));
 
             pzScan = pz + STRSIZE( zCheckList );
         }
@@ -282,7 +288,7 @@ skipToElseEnd( char* pzScan )
              *  Start processing the text.
              */
             char* pz = strchr( pzScan, '\n' );
-            if (pz != (char*)NULL)
+            if (pz != NULL)
                 return pz+1;
             return pzScan + strlen( pzScan );
         }
@@ -320,7 +326,7 @@ skipToElseEnd( char* pzScan )
  *  needs or may take arguments (e.g. '#define'), then there should
  *  also be an 'arg:' section describing the argument(s).
  */
-    STATIC char*
+STATIC char*
 doDir_IGNORE( char* pzArg, char* pzScan )
 {
     return pzScan;
@@ -345,7 +351,7 @@ doDir_IGNORE( char* pzArg, char* pzScan )
  *  After the definitions file has been processed, any remaining entries
  *  in the define list will be added to the environment.
 =*/
-    STATIC char*
+STATIC char*
 doDir_define( char* pzArg, char* pzScan )
 {
     char*  pzName = pzArg;
@@ -409,11 +415,12 @@ doDir_define( char* pzArg, char* pzScan )
  *  otherwise it will generate an error.
  *  It will be ignored.
 =*/
-    STATIC char*
+STATIC char*
 doDir_elif( char* pzArg, char* pzScan )
 {
-    fputs( "`#elif' directive encountered out of context\n", stderr );
-    AG_ABEND;
+    AG_ABEND( asprintf( "`#elif' directive encountered out of context"
+                        "\tin %s on line %d\n",
+                        pCurCtx->pzFileName, pCurCtx->lineNo ));
 }
 
 
@@ -424,13 +431,12 @@ doDir_elif( char* pzArg, char* pzScan )
  *  If it follows the @code{#if}, then it will be ignored.  Otherwise,
  *  it will change the processing state to the reverse of what it was.
 =*/
-    STATIC char*
+STATIC char*
 doDir_else( char* pzArg, char* pzScan )
 {
-    if (--ifdefLevel < 0) {
-        fprintf( stderr, zNoMatch, pCurCtx->pzFileName, "else" );
-        AG_ABEND;
-    }
+    if (--ifdefLevel < 0)
+        AG_ABEND( asprintf( zNoMatch, pCurCtx->pzFileName,
+                            pCurCtx->lineNo, "else" ));
 
     return skipToEndif( pzScan );
 }
@@ -442,13 +448,12 @@ doDir_else( char* pzArg, char* pzScan )
  *  This must follow an @code{#if}, @code{#ifdef} or @code{#ifndef}.
  *  In all cases, this will resume normal processing of text.
 =*/
-    STATIC char*
+STATIC char*
 doDir_endif( char* pzArg, char* pzScan )
 {
-    if (--ifdefLevel < 0) {
-        fprintf( stderr, zNoMatch, pCurCtx->pzFileName, "endif" );
-        AG_ABEND;
-    }
+    if (--ifdefLevel < 0)
+        AG_ABEND( asprintf( zNoMatch, pCurCtx->pzFileName,
+                            pCurCtx->lineNo, "endif" ));
 
     return pzScan;
 }
@@ -459,17 +464,15 @@ doDir_endif( char* pzArg, char* pzScan )
  *  text:
  *  Ends the text processed by a command shell into autogen definitions.
 =*/
-    STATIC char*
+STATIC char*
 doDir_endshell( char* pzArg, char* pzScan )
 {
     /*
      *  In actual practice, the '#endshell's must be consumed inside
      *  the 'doDir_shell()' procedure.
      */
-    tSCC zBadEnd[] = "ERROR:  in file %s, '#endshell' directive found "
-                     "without a matching '#shell'\n";
-    fprintf( stderr, zBadEnd, pCurCtx->pzFileName );
-    AG_ABEND;
+    AG_ABEND( asprintf( zNoMatch, pCurCtx->pzFileName,
+                        pCurCtx->lineNo, "endshell" ));
 }
 
 
@@ -481,11 +484,11 @@ doDir_endshell( char* pzArg, char* pzScan )
  *  This directive will cause AutoGen to stop processing
  *  and exit with a status of EXIT_FAILURE.
 =*/
-    STATIC char*
+STATIC char*
 doDir_error( char* pzArg, char* pzScan )
 {
-    fprintf( stderr, "#error directive -- %s\n", pzArg );
-    AG_ABEND;
+    AG_ABEND( asprintf( "#error directive -- in %s on line %d\n\t%s\n",
+                        pCurCtx->pzFileName, pCurCtx->lineNo, pzArg ));
 }
 
 
@@ -503,7 +506,7 @@ doDir_error( char* pzArg, char* pzScan )
  *  @code{#if} expressions are not analyzed.  @strong{Everything} from here
  *  to the matching @code{#endif} is skipped.
 =*/
-    STATIC char*
+STATIC char*
 doDir_if( char* pzArg, char* pzScan )
 {
     return skipToEndif( pzScan );
@@ -519,10 +522,10 @@ doDir_if( char* pzArg, char* pzScan )
  *  processed only if there is a corresponding @code{-Dname} command line
  *  option.
 =*/
-    STATIC char*
+STATIC char*
 doDir_ifdef( char* pzArg, char* pzScan )
 {
-    if (getDefine( pzArg ) == (char*)NULL)
+    if (getDefine( pzArg ) == NULL)
         return skipToElseEnd( pzScan );
     ifdefLevel++;
     return pzScan;
@@ -538,10 +541,10 @@ doDir_ifdef( char* pzArg, char* pzScan )
  *  processed only if there is @strong{not} a corresponding @code{-Dname}
  *  command line option or there was a canceling @code{-Uname} option.
 =*/
-    STATIC char*
+STATIC char*
 doDir_ifndef( char* pzArg, char* pzScan )
 {
-    if (getDefine( pzArg ) != (char*)NULL)
+    if (getDefine( pzArg ) != NULL)
         return skipToElseEnd( pzScan );
     ifdefLevel++;
     return pzScan;
@@ -558,7 +561,7 @@ doDir_ifndef( char* pzArg, char* pzScan )
  *  double quotes or angle brackets (as in a C program), then the
  *  include is ignored.
 =*/
-    STATIC char*
+STATIC char*
 doDir_include( char* pzArg, char* pzScan )
 {
     tSCC*      apzSfx[] = { "def", NULL };
@@ -576,7 +579,7 @@ doDir_include( char* pzArg, char* pzScan )
 
     if (! SUCCESSFUL( findFile( pzArg, zFullName, apzSfx ))) {
         tSCC zFmt[] = "WARNING:  cannot find `%s' definitions file\n";
-        fprintf( stderr, zFmt, pzArg );
+        fprintf( pfTrace, zFmt, pzArg );
         return pzScan;
     }
 
@@ -587,12 +590,12 @@ doDir_include( char* pzArg, char* pzScan )
     {
         struct stat stbf;
         if (stat( zFullName, &stbf ) != 0) {
-            fprintf( stderr, "WARNING %d (%s):  cannot stat `%s' "
+            fprintf( pfTrace, "WARNING %d (%s):  cannot stat `%s' "
                      "for include\n", errno, strerror( errno ), zFullName );
             return pzScan;
         }
         if (! S_ISREG( stbf.st_mode )) {
-            fprintf( stderr, "WARNING:  `%s' must be regular file to "
+            fprintf( pfTrace, "WARNING:  `%s' must be regular file to "
                      "include\n", zFullName );
             return pzScan;
         }
@@ -635,22 +638,16 @@ doDir_include( char* pzArg, char* pzScan )
         FILE*  fp = fopen( zFullName, "r" FOPEN_TEXT_FLAG );
         char*  pz = pzScan;
 
-        if (fp == (FILE*)NULL) {
-            fprintf( stderr, "%s ERROR %d (%s):  Cannot open `%s'\n",
-                     pzProg, errno, strerror( errno ),
-                     zFullName );
-            AG_ABEND;
-        }
+        if (fp == NULL)
+            AG_ABEND( asprintf( zCannot, errno, "open file", zFullName,
+                                strerror( errno )));
 
         do  {
             size_t rdct = fread( (void*)pz, 1, inclSize, fp );
 
-            if (rdct == 0) {
-                fprintf( stderr, "%s ERROR %d (%s):  Cannot read `%s'\n",
-                         pzProg, errno,
-                         strerror( errno ), zFullName );
-                break;
-            }
+            if (rdct == 0)
+                AG_ABEND( asprintf( zCannot, errno, "read file", zFullName,
+                                    strerror( errno )));
 
             pz += rdct;
             inclSize -= rdct;
@@ -674,7 +671,7 @@ doDir_include( char* pzArg, char* pzScan )
  *  file and approximate line number of any errors found in extracted
  *  definitions.
 =*/
-    STATIC char*
+STATIC char*
 doDir_line( char* pzArg, char* pzScan )
 {
     /*
@@ -697,7 +694,7 @@ doDir_line( char* pzArg, char* pzScan )
         return pzScan;
     {
         char* pz = strchr( pzArg, '"' );
-        if (pz == (char*)NULL)
+        if (pz == NULL)
             return pzScan;
         *pz = NUL;
     }
@@ -722,7 +719,7 @@ doDir_line( char* pzArg, char* pzScan )
  *  process that handles the back-quoted @code{`} text.
  *  @strong{CAUTION}@:  let not your @code{$SHELL} be @code{csh}.
 =*/
-    STATIC char*
+STATIC char*
 doDir_shell( char* pzArg, char* pzScan )
 {
     tSCC       zShellText[] = "Computed Definitions";
@@ -735,7 +732,7 @@ doDir_shell( char* pzArg, char* pzScan )
      *  The output time will always be the current time.
      *  The dynamic content is always current :)
      */
-    outTime = time( (time_t*)NULL );
+    outTime = time( NULL );
 
     /*
      *  IF there are no data after the '#shell' directive,
@@ -746,11 +743,10 @@ doDir_shell( char* pzArg, char* pzScan )
         return pzScan;
 
     pzScan = strstr( pzScan, zEndShell );
-    if (pzScan == (char*)NULL) {
-        fprintf( stderr, "%s ERROR:  '#endshell' directive not found "
-                 "after '#shell' directive\n", pzProg );
-        AG_ABEND;
-    }
+    if (pzScan == NULL)
+        AG_ABEND( asprintf( "Missing #endshell after '#shell' "
+                            "in %s on line %d\n", pCurCtx->pzFileName,
+                            pCurCtx->lineNo ));
 
     *pzScan = NUL;
 
@@ -760,7 +756,7 @@ doDir_shell( char* pzArg, char* pzScan )
      *  THEN the scan will resume on a zero-length string.
      */
     pzScan = strchr( pzScan + STRSIZE( zEndShell ), '\n' );
-    if (pzScan == (char*)NULL)
+    if (pzScan == NULL)
         pzScan = "";
 
     /*
@@ -773,7 +769,7 @@ doDir_shell( char* pzArg, char* pzScan )
      *  "file text" that is used for more definitions.
      */
     pzText = runShell( pzText );
-    if (  (pzText == (char*)NULL)
+    if (  (pzText == NULL)
        || (*pzText == NUL))
         return pzScan;
 
@@ -813,7 +809,7 @@ doDir_shell( char* pzArg, char* pzScan )
  *  Will remove any entries from the define list
  *  that match the undef name pattern.
 =*/
-    STATIC char*
+STATIC char*
 doDir_undef( char* pzArg, char* pzScan )
 {
     SET_OPT_UNDEFINE( pzArg );
